package packed

import (
	"archive/zip"
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"image"
	"image/jpeg"
	"image/png"
	"io/ioutil"
	"net/url"
	"os"
	"path"
	"strings"

	"ppt_server/app/model"
	"ppt_server/library"

	"github.com/gogf/gf/frame/g"
	"github.com/xxjwxc/gowp/workpool"
)


// 解析URL返回基本信息
type ObtainFile struct {
	url         string //上传URL
	fileName    string //上传文件名
	NameHash    string //文件名hash
	ContentHash string //内容hash
	DocType     string //doc_type
	MineType    string //文件mine类型
	fileInfoURL string //解析文件URL地址
	uploadPath  string //上传URL路径
	DataPath    string // 数据库存储Path
	Link        string //数据库存储link
	FileSize    int64
	PageCount   int
	bucket      *ossSdk
	ok          chan bool  //上传传课件状态
	err         chan error // 上传课件失败Error
}

func ParserURL(r *model.FileUploadRequest, errChan chan error, ok chan bool) (*ObtainFile, error) {
	u, err := url.Parse(r.Url)
	if err != nil {
		g.Log().Async().Error(err)
		return nil, err
	}

	dataPath := u.EscapedPath()[1:]
	slicePath := strings.Split(dataPath, "/")

	if len(slicePath) < 4 {
		g.Log().Async().Error(errors.New("上传URL格式不正确"))
		return nil, errors.New("上传URL格式不正确")
	}
	f := &file{
		ext: path.Ext(r.Name),
	}

	buecket, err := NewOss("xiaoyu-live")
	if err != nil {
		g.Log().Async().Error(err)
		return nil, err
	}

	file := &ObtainFile{
		url:         r.Url,
		fileName:    r.Name,
		NameHash:    slicePath[3],
		ContentHash: slicePath[2],
		DocType:     f.getDocType(),
		MineType:    f.getMine(),
		bucket:      buecket,
		fileInfoURL: "http://doc.offcncloud.com/?info=0&ssl=1&furl=" + g.Cfg().GetString("oss.url") + u.EscapedPath(),
		uploadPath:  strings.Join(slicePath[0:len(slicePath)-1], "/") + "/",
		DataPath:    dataPath,
		err:         errChan,
		ok:          ok,
		Link:        strings.Replace(r.Url, "xiaoyu-live.oss-cn-beijing-internal.aliyuncs.com", "desktop.offcncloud.com", -1),
	}

	return file, nil
}

//从web365获取上传文件信息
func (f *ObtainFile) fileInfo() error {

	fileBytes, err := library.Get(f.fileInfoURL)
	if err != nil {
		return err
	}
	var fileInfo model.UploadFileInfo
	if err := json.Unmarshal(fileBytes, &fileInfo); err != nil {
		return err
	}

	if fileInfo.FileSize != 0 {
		f.FileSize = fileInfo.FileSize
	}
	switch {
	case fileInfo.SlideCount != 0:
		f.PageCount = fileInfo.SlideCount
	case fileInfo.PageCount != 0:
		f.PageCount = fileInfo.PageCount
	case fileInfo.SheetCount != 0:
		f.PageCount = fileInfo.SheetCount
	}

	return nil
}

func (f *ObtainFile) job(file *zip.File) ([]interface{}, error) {
	fd, err := file.Open()
	if err != nil {
		return nil, err
	}

	defer fd.Close()

	reqBody := []byte{}
	reqBody, _ = ioutil.ReadAll(fd)

	// 获取图片信息reader
	imageReader := ioutil.NopCloser(bytes.NewBuffer(reqBody))
	// 上传图片reader
	fileReader := ioutil.NopCloser(bytes.NewBuffer(reqBody))

	var img image.Image
	fileSuffix := path.Ext(file.Name) //获取文件后缀
	switch fileSuffix {
	case ".png":
		img, _ = png.Decode(imageReader)
	case ".jpeg":
		img, _ = jpeg.Decode(imageReader)
	default:
		return nil, errors.New("image ext is not found")
	}
	if img == nil {
		return nil, errors.New("Image invalid")
	}

	var filenameOnly string
	filenameOnly = strings.TrimRight(file.Name, fileSuffix)
	filenameOnly = strings.Replace(filenameOnly, "p", "", -1)
	fileName := fmt.Sprintf("%05s", filenameOnly) + fileSuffix

	info := img.Bounds()
	var imageInfo []interface{}
	imageInfo = append(imageInfo, fileName)
	imageInfo = append(imageInfo, info.Max.X)
	imageInfo = append(imageInfo, info.Max.Y)

	g.Log().Async().Infof("Upload picture file: %s Success", fileName)

	return imageInfo, f.bucket.Upload(f.uploadPath+"images/"+fileName, fileReader)
}

func (f *ObtainFile) uploadPng(zipFileAddress string) ([][]interface{}, error) {
	reader, err := zip.OpenReader(zipFileAddress)
	if err != nil {
		return nil, err
	}
	defer reader.Close()
	pool := workpool.New(100)
	var imageInfos [][]interface{}
	for _, file := range reader.File {
		pool.DoWait(func() error {
			var err error
			var imageInfo []interface{}
			for i := 0; i < 3; i++ {
				if imageInfo, err = f.job(file); err == nil {
					imageInfos = append(imageInfos, imageInfo)
					break
				}
			}
			return err
		})
	}
	if err = pool.Wait(); err != nil {
		return nil, err
	}
	return imageInfos, nil
}

// 上传packed.json

func (f *ObtainFile) uploadPacked(imagesInfos [][]interface{}) error {
	packed := model.Packed{
		FileName:      f.fileName,
		Hash:          f.ContentHash,
		OssImagesPath: f.uploadPath + "images/",
		ImageInfos:    imagesInfos,
	}

	for i := 1; i <= f.PageCount; i++ {
		packed.ImagesName = append(packed.ImagesName, fmt.Sprintf("%05d", i)+".png")
	}
	data, err := json.Marshal(packed)
	if err != nil {
		return err
	}
	return f.bucket.Upload(f.uploadPath+"packed.json", bytes.NewReader(data))
}

// 进行上传操作
func (f *ObtainFile) Worker(err error) {

	if err != nil {
		return
	}
	// 解析课件获取课件信息
	if err = f.fileInfo(); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s file info complete", f.fileName)

	zipURL := fmt.Sprintf("http://doc.offcncloud.com/?info=1&words=%v&ssl=1&furl=%s",
		f.PageCount, g.Cfg().GetString("oss.url")+"/"+f.DataPath)

	// 下载ZIP包
	zipURLAddress := "D:/" + f.ContentHash + ".zip"
	if err = library.Download(zipURL, zipURLAddress); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s Zip Donwload Complete", f.fileName)

	// 解析ZIP包并上传图片到OSS
	imagesInfos, err := f.uploadPng(zipURLAddress)
	if err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s Upload picture Complete!", f.fileName)
	// 上传packed.json到oss
	if err := f.uploadPacked(imagesInfos); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}

	g.Log().Async().Infof("filename == %s upload success!", f.fileName)
	// 清理zip包
	_ = os.Remove(zipURLAddress)
	f.ok <- true
}
